@@ -1,42 +1,6 @@
-# Copyright (C) 2014 - 2018  Jack O. Wasey
-#
-# This file is part of icd.
-#
-# icd is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# icd is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with icd. If not, see <http:#www.gnu.org/licenses/>.
-
-#' @title Convert between and identify 'long' and 'wide' patient data formats
-#' @description Long and Wide Formats: As is common with many data sets, key
-#'   variables can be concentrated in one column or spread over several. Tools
-#'   format of clinical and administrative hospital data, we can perform the
-#'   conversion efficiently and accurately, while keeping some metadata about
-#'   the codes intact, e.g. whether they are ICD-9 or ICD-10.
-#' @details Long or wide format ICD data is expected to be in a data frame. It
-#'   does not carry any ICD classes at the top level, even if it only contains
-#'   one type of code, but its constituent columns may have a class specified,
-#'   e.g. 'icd9'.
-#' @param x \code{data.frame} or \code{matrix} to set class, or convert.
-#' @param ... arguments passed on to create a \code{data.frame}
-#' @description \code{icd_long_data} and \code{icd_wide_data} create
-#'   \code{data.frame}s using all the arguments, and sets the class, whereas
-#'   \code{as.icd_long_data} and \code{as.icd_wide_data} just set the class of
-#'   an existing \code{data.frame}.
-#' @name icd_long_data
-NULL
-
-#' Convert ICD9 codes between formats and structures.
+#' Convert ICD data between formats and structures.
 #'
-#' ICD-9 codes are represented in \emph{short} and \emph{decimal} forms. The
+#' ICD codes are represented in \emph{short} and \emph{decimal} forms. The
 #' short form has up to 5 digits, or V or E followed by up to four digits. The
 #' decimal form has a decimal point to delimit the top-level (henceforth
 #' \emph{major}) category, and the \emph{minor} part containing the subsidiary
@@ -48,13 +12,13 @@ NULL
 #'
 #' \pkg{icd} does not covert ICD-9 to ICD-10 codes yet.
 #'
-#' @family ICD code conversion
+#' @family ICD data conversion
 #' @name convert
 NULL
 
-#' convert the chapter headings to lists of codes
+#' Convert chapters to lists of codes for use as a comorbidity map
 #'
-#' the chapter headings can be converted into the full set of their children,
+#' The chapter headings can be converted into the full set of their children,
 #' and then used to look-up which chapter, sub-chapter, or 'major' a given code
 #' belongs. Always returns a map with short-form ICD-9 codes. These can be
 #' converted in bulk with \code{lapply} and \code{short_to_decimal}.
@@ -64,27 +28,30 @@ NULL
 #'   officially defined ICD-9 (currently ICD-9-CM) codes will be used in the
 #'   expansion, not any syntactically possible ICD-9 code.
 #' @keywords internal manip
-icd9_chapters_to_map <- function(x, defined = FALSE) {
-  if (is.character(x) && exists(x))
-    x <- get(x)
-  assert_list(x, types = "character", min.len = 1, names = "unique")
+#' @export
+chapters_to_map <- function(x, defined = TRUE) {
+  if (is.character(x) && exists(x)) x <- get(x)
+  stopifnot(is.list(x))
+  stopifnot(all(vapply(x, is.character, logical(1))))
   ranges <- names(x)
   map <- list()
   for (r in ranges) {
-    map[[r]] <- expand_range.icd9(x[[r]][1], x[[r]][2],
-                                  short_code = TRUE, defined = defined)
+    map[[r]] <- expand_range(x[[r]][1],
+      x[[r]][2],
+      short_code = TRUE,
+      defined = defined
+    )
   }
   map
 }
 
 #' Convert ICD data from wide to long format
-#'
-#' Reshaping data is a common task, and is made easier here by knowing more
-#' about the underlying structure of the data. This function wraps the
-#' \code{\link[stats]{reshape}} function with specific behavior and checks
-#' related to ICD codes. Empty strings and NA values will be dropped, and
-#' everything else kept. No validation of the ICD codes is done.
-#'
+#' @template widevlong
+#' @details Reshaping data is a common task, and is made easier here by knowing
+#'   more about the underlying structure of the data. This function wraps the
+#'   \code{\link[stats]{reshape}} function with specific behavior and checks
+#'   related to ICD codes. Empty strings and NA values will be dropped, and
+#'   everything else kept. No validation of the ICD codes is done.
 #' @param x \code{data.frame} in wide format, i.e. one row per patient, and
 #'   multiple columns containing ICD codes, empty strings or NA.
 #' @template visit_name
@@ -99,45 +66,48 @@ icd9_chapters_to_map <- function(x, defined = FALSE) {
 #'   a column named by \code{icd.name} containing all the non-NA and non-empty
 #'   codes found in the wide input data.
 #' @examples
-#' widedf <- data.frame(visit_name = c("a", "b", "c"),
+#' widedf <- data.frame(
+#'   visit_name = c("a", "b", "c"),
 #'   icd9_01 = c("441", "4424", "441"),
 #'   icd9_02 = c(NA, "443", NA)
-#'   )
+#' )
 #' wide_to_long(widedf)
-#' @family ICD code conversion
+#' @family ICD data conversion
 #' @export
 wide_to_long <- function(x,
                          visit_name = get_visit_name(x),
                          icd_labels = NULL,
                          icd_name = "icd_code",
                          icd_regex = c("icd", "diag", "dx_", "dx")) {
-  assert_data_frame(x, min.rows = 1, min.cols = 2)
+  stopifnot(is.data.frame(x), nrow(x) > 0, ncol(x) >= 2)
   # explicitly make a data frame so tibble works (and maybe data.table, too?)
   x <- as.data.frame(x)
   assert_string(visit_name)
-  assert(check_null(icd_labels),
-         check_character(icd_labels, min.len = 1))
+  stopifnot(is.null(icd_labels) || is.character(icd_labels))
   assert_string(icd_name)
-  assert_character(icd_regex, min.chars = 1, any.missing = FALSE, min.len = 1)
-
+  stopifnot(is.character(icd_regex), nchar(icd_regex) > 0)
   if (is.null(icd_labels)) {
     re <- length(icd_regex)
     while (re > 0) {
       icd_labels <- grep(rev(icd_regex)[re], names(x),
-                         ignore.case = TRUE, value = TRUE)
+        ignore.case = TRUE, value = TRUE
+      )
       if (length(icd_labels)) break
       re <- re - 1
     }
   }
-  assert_character(icd_labels, any.missing = FALSE, min.chars = 1,
-                   min.len = 1, max.len = ncol(x) - 1)
+  assert_character(icd_labels,
+    any.missing = FALSE, min.chars = 1,
+    min.len = 1, max.len = ncol(x) - 1
+  )
   stopifnot(all(icd_labels %in% names(x)))
   res <- stats::reshape(x,
-                        direction = "long",
-                        varying = icd_labels,
-                        idvar = visit_name,
-                        timevar = NULL,
-                        v.names = icd_name)
+    direction = "long",
+    varying = icd_labels,
+    idvar = visit_name,
+    timevar = NULL,
+    v.names = icd_name
+  )
   res <- res[!is.na(res[[icd_name]]), ]
   res <- res[nchar(as_char_no_warn(res[[icd_name]])) > 0, ]
   res <- res[order(res[[visit_name]]), ]
@@ -146,9 +116,10 @@ wide_to_long <- function(x,
 }
 
 #' Convert ICD data from long to wide format
-#'
-#' This is more complicated than reshape or \code{reshape2::dcast} allows. This
-#' is a reasonably simple solution using built-in functions.
+#' @template widevlong
+#' @details This is more complicated than expected using \code{base::reshape} or
+#'   \code{reshape2::dcast} allows. This is a reasonably simple solution using
+#'   built-in functions.
 #' @param x data.frame of long-form data, one column for visit_name and one for
 #'   ICD code
 #' @template visit_name
@@ -157,63 +128,59 @@ wide_to_long <- function(x,
 #' @param min_width, single integer, if specified, writes out this many columns
 #'   even if no patients have that many codes. Must be greater than or equal to
 #'   the maximum number of codes per patient.
-#' @param aggr single logical value, if TRUE (the default) will take more time
-#'   to find out-of-order visit_names, and combine all the codes for each unique
-#'   visit_name. If \code{FALSE}, then out-of-order visit_names will result in a
-#'   row in the output data per contiguous block of identical visit_names.
-#' @param return_df single logical value, if \code{TRUE}, return a data frame
-#'   with a field for the visit_name. This may be more convenient, but the
-#'   default of \code{FALSE} gives the more natural return data of a matrix with
-#'   row names being the visit IDs from \code{visit_name}s.
 #' @examples
-#'   longdf <- data.frame(visit_name = c("a", "b", "b", "c"),
-#'     icd9 = c("441", "4424", "443", "441"))
-#'   long_to_wide(longdf)
-#'   long_to_wide(longdf, prefix = "ICD10_")
-#' @family ICD-9 convert
-#' @family ICD code conversion
+#' longdf <- data.frame(
+#'   visit_name = c("a", "b", "b", "c"),
+#'   icd9 = c("441", "4424", "443", "441")
+#' )
+#' long_to_wide(longdf)
+#' long_to_wide(longdf, prefix = "ICD10_")
+#' @family ICD data conversion
 #' @export
 long_to_wide <- function(x,
                          visit_name = get_visit_name(x),
                          icd_name = get_icd_name(x),
                          prefix = "icd_",
-                         min_width = 0,
-                         aggr = TRUE,
-                         return_df = FALSE) {
-
+                         min_width = 1L) {
   assert_data_frame(x, col.names = "unique")
   visit_name <- as_char_no_warn(visit_name)
   icd_name <- as_char_no_warn(icd_name)
-  assert_string(prefix, min.chars = 1)
-  assert_count(min_width, na.ok = FALSE)
-  assert_flag(aggr)
-  assert_flag(return_df)
-
-  # we're now going to return a matrix
-  icd9_name_was_factor <- is.factor(x[[visit_name]]) # TODO: visit or icd name?
-  if (icd9_name_was_factor) iv_levels <- levels(x[[visit_name]])
-  x[[visit_name]] <- as_char_no_warn(x[[visit_name]])
-  x[[icd_name]] <- as_char_no_warn(x[[icd_name]])
-  mat <- long_to_wide_cpp(x, visitId = visit_name,
-                          icd9Field = icd_name, aggregate = aggr)
-  if (!return_df)
-    return(as.icd_wide_data(mat))
-  if (icd9_name_was_factor)
-    rownm <- factor(x = rownames(mat), levels = iv_levels)
-  else
-    rownm <- rownames(mat)
-  df.out <- cbind(rownm, as.data.frame(unname(mat)),
-                  stringsAsFactors = icd9_name_was_factor)
-  names(df.out)[1] <- visit_name
-  # perhaps leave (duplicated) rownames which came from the matrix:
-  rownames(df.out) <- NULL
-  nc <- ncol(df.out) - 1
+  stopifnot(visit_name %in% names(x))
+  stopifnot(icd_name %in% names(x))
+  stopifnot(is.character(prefix), length(prefix) == 1, nchar(prefix) > 0)
+  stopifnot(all(min_width > 0), length(min_width) == 1)
+  visit_name_f <- is.factor(x[[visit_name]])
+  icd_name_f <- is.factor(x[[icd_name]])
+  if (icd_name_f) i_levels <- levels(x[[icd_name]])
+  x_order <- order(x[[visit_name]])
+  x <- x[x_order, ]
+  if (visit_name_f) {
+    v_levels <- levels(x[[visit_name]])
+  } else {
+    v_levels <- x[!duplicated(x[[visit_name]]), visit_name]
+  }
+  x$seqnum <- sequence(tabulate(factor(x[[visit_name]], levels = v_levels)))
+  out <- reshape(x[c(visit_name, "seqnum", icd_name)],
+    idvar = visit_name,
+    timevar = "seqnum",
+    direction = "wide"
+  )
+  if (visit_name_f) {
+    out[[visit_name]] <- factor(x = out[[visit_name]], levels = v_levels)
+  }
+  if (icd_name_f) {
+    out[-which(names(out) == visit_name)] <-
+      lapply(out[-which(names(out) == visit_name)], factor, levels = i_levels)
+  }
+  nc <- ncol(out) - 1
+  names(out)[-which(names(out) == visit_name)] <-
+    paste(prefix, sprintf("%03d", 1:nc), sep = "")
   if (nc < min_width) {
-    df.out <- cbind(df.out, matrix(rep(NA, min_width - nc), nrow = 1))
+    out <- cbind(out, matrix(NA, ncol = min_width - nc))
     nc <- min_width
   }
-  names(df.out)[-1] <- paste(prefix, sprintf("%03d", 1:nc), sep = "")
-  as.icd_wide_data(df.out)
+  names(out)[-1] <- paste(prefix, sprintf("%03d", 1:nc), sep = "")
+  as.icd_wide_data(out, warn = FALSE)
 }
 
 #' convert comorbidity data frame from matrix
@@ -227,8 +194,9 @@ long_to_wide <- function(x,
 #' @template stringsAsFactors
 #' @examples
 #' longdf <- icd_long_data(
-#'             visit_id = c("a", "b", "b", "c"),
-#'             icd9 = as.icd9(c("441", "4240", "443", "441")))
+#'   visit_id = c("a", "b", "b", "c"),
+#'   icd9 = as.icd9(c("441", "4240", "443", "441"))
+#' )
 #' mat <- icd9_comorbid_elix(longdf)
 #' class(mat)
 #' typeof(mat)
@@ -242,11 +210,15 @@ long_to_wide <- function(x,
 #' # the system-wide option you may have set e.g. with
 #' # options("stringsAsFactors" = FALSE).
 #' is.factor(df.out[["visit_id"]])
+#' @family ICD data conversion
+#' @seealso \code{\link{comorbid_df_to_mat}}
 #' @export
 comorbid_mat_to_df <- function(x, visit_name = "visit_id",
                                stringsAsFactors = getOption("stringsAsFactors")) { # nolint
-  assert_matrix(x, min.rows = 1, min.cols = 1,
-                row.names = "named", col.names = "named")
+  assert_matrix(x,
+    min.rows = 1, min.cols = 1,
+    row.names = "named", col.names = "named"
+  )
   assert_string(visit_name)
   assert_flag(stringsAsFactors) # nolint
   out <- data.frame(rownames(x), x, stringsAsFactors = stringsAsFactors, row.names = NULL) # nolint
@@ -266,15 +238,17 @@ comorbid_mat_to_df <- function(x, visit_name = "visit_id",
 #' @template stringsAsFactors
 #' @examples
 #' longdf <- icd_long_data(
-#'             visit = c("a", "b", "b", "c"),
-#'             icd9 = c("441", "4240", "443", "441")
-#'             )
+#'   visit = c("a", "b", "b", "c"),
+#'   icd9 = c("441", "4240", "443", "441")
+#' )
 #' cmbdf <- icd9_comorbid_elix(longdf, return_df = TRUE)
 #' class(cmbdf)
 #' rownames(cmbdf)
 #' mat.out <- comorbid_df_to_mat(cmbdf)
 #' stopifnot(is.matrix(mat.out))
 #' mat.out[, 1:4]
+#' @family ICD data conversion
+#' @seealso \code{\link{comorbid_mat_to_df}}
 #' @export
 comorbid_df_to_mat <- function(x, visit_name = get_visit_name(x),
                                stringsAsFactors = getOption("stringsAsFactors")) { # nolint
@@ -289,12 +263,14 @@ comorbid_df_to_mat <- function(x, visit_name = get_visit_name(x),
 
 #' Convert ICD codes from short to decimal forms
 #'
-#' Convert codes between short and decimal forms
+#' Convert from short to decimal forms of ICD codes.
 #' @param x ICD codes
 #' @export
-#' @keywords internal
-short_to_decimal <- function(x)
+#' @keywords manip
+#' @family ICD data conversion
+short_to_decimal <- function(x) {
   UseMethod("short_to_decimal")
+}
 
 #' @describeIn short_to_decimal convert ICD codes of unknown type from short
 #'   to decimal format
@@ -313,15 +289,16 @@ short_to_decimal.default <- function(x) {
 #'   format
 #' @export
 #' @keywords internal manip
-short_to_decimal.icd9 <- function(x)
-  icd9(as.decimal_diag(icd9_short_to_decimal_cpp(x)))
+short_to_decimal.icd9 <- function(x) {
+  icd9(as.decimal_diag(icd9_short_to_decimal_rcpp(x)))
+}
 
 #' @describeIn short_to_decimal convert ICD-10 codes from short to decimal
 #'   format
 #' @export
 #' @keywords internal manip
 short_to_decimal.icd10 <- function(x) {
-  x <- trim(x)
+  x <- trimws(x)
   out <- substr(x, 0, 3) # majors
   minors <- substr(x, 4, 100L)
   out[minors != ""] <- paste0(out[minors != ""], ".", minors[minors != ""])
@@ -332,17 +309,20 @@ short_to_decimal.icd10 <- function(x) {
 #'   format
 #' @export
 #' @keywords internal manip
-short_to_decimal.icd10cm <- function(x)
+short_to_decimal.icd10cm <- function(x) {
   as.icd10cm(short_to_decimal.icd10(x))
+}
 
 #' Convert Decimal format ICD codes to short format
 #'
 #' This usually just entails removing the decimal point, but also does some
-#' limited validation and tidying up.
+#' limited validation and tidying up. Missing leading zeroes will be added for
+#' correctness of the shortened codes.
 #'
 #' @param x ICD codes
 #' @export
-#' @keywords internal manip
+#' @keywords manip
+#' @family ICD data conversion
 decimal_to_short <- function(x) {
   UseMethod("decimal_to_short")
 }
@@ -353,10 +333,10 @@ decimal_to_short <- function(x) {
 #' @keywords internal manip
 decimal_to_short.icd9 <- function(x) {
   if (is.factor(x)) {
-    levels(x) <- icd9(as.short_diag(icd9_decimal_to_short_cpp(levels(x))))
+    levels(x) <- icd9(as.short_diag(icd9_decimal_to_short_rcpp(levels(x))))
     return(x)
   }
-  icd9(as.short_diag(icd9_decimal_to_short_cpp(x)))
+  icd9(as.short_diag(icd9_decimal_to_short_rcpp(x)))
 }
 
 #' @describeIn decimal_to_short convert ICD-10 codes from decimal to short
@@ -391,54 +371,88 @@ decimal_to_short.default <- function(x) {
   )
 }
 
-#' Convert decimal ICD codes to component parts
-#' @export
-#' @keywords internal
-decimal_to_parts <- function(x, mnr_empty = "")
-  UseMethod("decimal_to_parts")
-
 #' Convert short format ICD codes to component parts
-#' @export
 #' @keywords internal manip
-short_to_parts <- function(x, mnr_empty = "")
+#' @noRd
+short_to_parts <- function(x, mnr_empty = "") {
   UseMethod("short_to_parts")
+}
 
-#' @describeIn short_to_parts Convert short format ICD-9 code to parts
+#' @describeIn short_to_parts Convert short format ICD-9 codes to parts
 #' @export
 #' @keywords internal manip
+#' @noRd
 short_to_parts.icd9 <- function(x, mnr_empty = "") {
   # Cannot specify default values in both header and C++ function body, so use a
   # shim here.
-  .Call("_icd_icd9ShortToPartsCpp", x, mnr_empty)
+  icd9ShortToParts(x, mnrEmpty = mnr_empty)
 }
 
-#' @describeIn decimal_to_parts Convert decimal ICD-9 code to parts
+#' @describeIn short_to_parts Convert short format ICD-10 codes to parts
 #' @export
 #' @keywords internal manip
-decimal_to_parts.icd9 <- function(x, mnr_empty = "")
-  .Call("_icd_icd9DecimalToPartsCpp", x, mnr_empty)
+#' @noRd
+short_to_parts.icd10 <- function(x, mnr_empty = "") {
+  icd10_short_to_parts_rcpp(x, mnrEmpty = mnr_empty)
+}
+
+#' @describeIn short_to_parts Convert short format ICD-10-CM codes to parts
+#' @export
+#' @keywords internal manip
+#' @noRd
+short_to_parts.icd10cm <- function(x, mnr_empty = "") {
+  icd10_short_to_parts_rcpp(x, mnrEmpty = mnr_empty)
+}
 
 #' @describeIn short_to_parts Convert short format ICD code to parts,
 #'   guessing whether ICD-9 or ICD-10
 #' @export
 #' @keywords internal manip
+#' @noRd
 short_to_parts.character <- function(x, mnr_empty = "")
-  # No default values in header plus C++ function body, so shim here.
+# No default values in header plus C++ function body, so shim here.
   switch(
     guess_version(x, short_code = TRUE),
-    "icd9" = .Call("_icd_icd9ShortToPartsCpp", x, mnr_empty),
+    "icd9" = icd9ShortToParts(x, mnrEmpty = mnr_empty),
     "icd10" = short_to_parts.icd10(x, mnr_empty = mnr_empty),
     stop("Unknown ICD version guessed from input")
   )
 
+#' Convert decimal ICD codes to component parts
+#' @keywords internal manip
+#' @noRd
+decimal_to_parts <- function(x, mnr_empty = "") {
+  UseMethod("decimal_to_parts")
+}
+
+#' @describeIn decimal_to_parts Convert decimal ICD-9 code to parts
+#' @export
+#' @keywords internal manip
+#' @noRd
+decimal_to_parts.icd9 <- function(x, mnr_empty = "") {
+  icd9DecimalToParts(x, mnrEmpty = mnr_empty)
+}
+
 #' @describeIn decimal_to_parts Convert decimal ICD code to parts, guessing
 #'   ICD version
 #' @export
 #' @keywords internal manip
-decimal_to_parts.character <- function(x, mnr_empty = "")
+#' @noRd
+decimal_to_parts.character <- function(x, mnr_empty = "") {
   switch(
     guess_version(x, short_code = FALSE),
-    "icd9" = .Call("_icd_icd9DecimalToPartsCpp", x, mnr_empty),
+    "icd9" = icd9DecimalToParts(x, mnr_empty),
     "icd10" = decimal_to_parts.icd10(x, mnr_empty = mnr_empty),
     stop("Unknown ICD version guessed from input")
   )
+}
+
+#' @describeIn decimal_to_parts Convert decimal ICD-10 code to parts. This
+#'   shares almost 100% code with the ICD-9 version: someday combine the common
+#'   code.
+#' @export
+#' @keywords internal manip
+#' @noRd
+decimal_to_parts.icd10 <- function(x, mnr_empty = "") {
+  icd10DecimalToParts(x, mnrEmpty = mnr_empty)
+}
