@@ -1,36 +1,25 @@
-# Copyright (C) 2014 - 2018  Jack O. Wasey
-#
-# This file is part of icd.
-#
-# icd is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# icd is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with icd. If not, see <http:#www.gnu.org/licenses/>.
-
-#' extract alphabetic, and numeric part of ICD-9 code prefix
+#' Extract alphabetic, and numeric part of ICD-9 code prefix
 #'
-#' removes white space and separates V or E if present.
+#' Removes white space and separates a \sQuote{V} or \sQuote{E} if present.
 #' @template icd9-any
 #' @return vector or matrix, with first item of each row (or whole vector)
-#'   containing V, E or "". The second part contains the numeric parts of the
-#'   code, which may include a decimal point.
+#'   containing \sQuote{V}, \sQuote{E} or \sQuote{}. The second part contains
+#'   the numeric parts of the code, which may include a decimal point.
 #' @keywords internal manip
+#' @noRd
 icd9_extract_alpha_numeric <- function(x) {
-  assert(check_factor(x), check_character(x))
+  assert_fac_or_char(x)
   # generate list, then flip into a matrix with a row for each code, and the
   # alpha part in first column, and numeric part in the second
-  as_char_no_warn(x) %>%
-    str_match_all(pattern = "([VvEe]?)([[:digit:].]+)") %>%
-    vapply(FUN = function(y) matrix(data = y[2:3], nrow = 1, ncol = 2),
-           FUN.VALUE = c(NA_character_, NA_character_)) %>% t
+  t(
+    vapply(
+      .str_match_all(as_char_no_warn(x),
+        pattern = "([VvEe]?)([[:digit:].]+)"
+      ),
+      FUN = function(y) matrix(data = y[2:3], nrow = 1, ncol = 2),
+      FUN.VALUE = c(NA_character_, NA_character_)
+    )
+  )
 }
 
 #' drop zero padding from decimal ICD-9 code.
@@ -45,10 +34,11 @@ icd9_extract_alpha_numeric <- function(x) {
 #' @return character vector of ICD-9 codes with extra zeroes dropped from major
 #'   part
 #' @keywords internal manip
+#' @noRd
 icd9_drop_leading_zeroes <- function(x, short_code = guess_short(x)) {
-  assert(check_factor(x), check_character(x))
-  assert(check_null(short_code), check_flag(short_code))
-
+  assert_fac_or_char(x)
+  stopifnot(is.null(short_code) ||
+    (is.logical(short_code) && length(short_code) == 1L))
   if (short_code) {
     parts <- short_to_parts.icd9(x = x, mnr_empty = "")
     # very important: only drop the zero in V codes if the minor part is empty.
@@ -61,30 +51,36 @@ icd9_drop_leading_zeroes <- function(x, short_code = guess_short(x)) {
 }
 
 icd9_add_leading_zeroes <- function(x, short_code = guess_short(x)) {
-  assert(check_factor(x), check_character(x))
+  assert_fac_or_char(x)
   assert_flag(short_code)
   if (is.factor(x)) {
-    levels(x) <- icd9_add_leading_zeroes_cpp(levels(x), short_code)
+    levels(x) <- icd9_add_leading_zeroes_rcpp(levels(x), short_code)
     x
-  } else icd9_add_leading_zeroes_cpp(x, short_code)
+  } else {
+    icd9_add_leading_zeroes_rcpp(x, short_code)
+  }
 }
 
-#' @rdname icd9_drop_leading_zeroes
+#' Drop leading zeroes from major parts of ICD-9 codes
 #' @keywords internal manip
+#' @noRd
 icd9_drop_leading_zeroes_major <- function(major) {
   # (valid) E codes from 000 exist. Dropping zeroes from E000 would require a
   # lot of logic for no current benefit. Defer this until it is a problem.
-  major <- trim(major)
+  major <- trimws(major)
   # not checking validity, necessarily, just quick check
   is_v <- icd9_is_v(major)
   is_n <- icd9_is_valid_major_n(major)
   major[is_v] <- sub(
     pattern = "^[[:space:]]*([Vv])0([[:digit:]])[[:space:]]*$",
     replacement = "\\1\\2",
-    x = major[is_v])
-  #just replace the FIRST string of zeros everything else is passed through
-  major[is_n] <- sub(pattern = "^[[:space:]]*0{1,2}",
-                     replacement = "",
-                     x = major[is_n])
+    x = major[is_v]
+  )
+  # just replace the FIRST string of zeros everything else is passed through
+  major[is_n] <- sub(
+    pattern = "^[[:space:]]*0{1,2}",
+    replacement = "",
+    x = major[is_n]
+  )
   major
 }
