@@ -1,146 +1,280 @@
-# Copyright (C) 2014 - 2018  Jack O. Wasey
-#
-# This file is part of icd.
-#
-# icd is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# icd is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with icd. If not, see <http:#www.gnu.org/licenses/>.
-
 utils::globalVariables(
-  c("re_icd10_any", "re_icd10_decimal", "re_icd10_short",
-    "re_icd10cm_any", "re_icd10cm_decimal", "re_icd10cm_short", "re_icd10_major",
-    "re_icd10cm_major", "re_icd10_major_bare",
-    "re_icd10cm_major_bare", "re_icd9_any",
-    "re_icd9_short", "re_icd9_decimal_strict_bare", "re_icd9_decimal_bare",
-    "re_icd9_decimal", "re_icd9_any_e", "re_icd9_any_v", "re_icd9_any_n",
-    "re_icd9_short_e", "re_icd9_short_v", "re_icd9_short_n", "re_icd9_decimal_e",
-    "re_icd9_decimal_v", "re_icd9_decimal_n", "re_icd9_decimal_e_strict_bare",
-    "re_icd9_decimal_v_strict_bare", "re_icd9_decimal_n_strict_bare",
-    "re_icd9_decimal_e_bare", "re_icd9_decimal_v_bare", "re_icd9_decimal_n_bare",
-    "re_icd9_minor_e", "re_icd9_minor_nv", "re_icd9_major_strict_bare",
-    "re_icd9_major_strict", "re_icd9_major_bare", "re_icd9_major",
-    "re_icd9_major_e_strict", "re_icd9_major_e", "re_icd9_major_v_strict",
-    "re_icd9_major_v", "re_icd9_major_n_strict", "re_icd9_major_n"))
+  c(
+    "re_icd10_any",
+    "re_icd10_decimal",
+    "re_icd10_short",
+    "re_icd10cm_any",
+    "re_icd10cm_decimal",
+    "re_icd10cm_short",
+    "re_icd10_major",
+    "re_icd10cm_major",
+    "re_icd10_major_bare",
+    "re_icd10cm_major_bare",
+    "re_icd9_any",
+    "re_icd9_short",
+    "re_icd9_decimal_strict_bare",
+    "re_icd9_decimal_bare",
+    "re_icd9_decimal",
+    "re_icd9_any_e",
+    "re_icd9_any_v",
+    "re_icd9_any_n",
+    "re_icd9_short_e",
+    "re_icd9_short_v",
+    "re_icd9_short_n",
+    "re_icd9_decimal_e",
+    "re_icd9_decimal_v",
+    "re_icd9_decimal_n",
+    "re_icd9_decimal_e_strict_bare",
+    "re_icd9_decimal_v_strict_bare",
+    "re_icd9_decimal_n_strict_bare",
+    "re_icd9_decimal_e_bare",
+    "re_icd9_decimal_v_bare",
+    "re_icd9_decimal_n_bare",
+    "re_icd9_minor_e",
+    "re_icd9_minor_nv",
+    "re_icd9_major_strict_bare",
+    "re_icd9_major_strict",
+    "re_icd9_major_bare",
+    "re_icd9_major",
+    "re_icd9_major_e_strict",
+    "re_icd9_major_e",
+    "re_icd9_major_v_strict",
+    "re_icd9_major_v",
+    "re_icd9_major_n_strict",
+    "re_icd9_major_n"
+  )
+)
 
 # nolint start
 
 # Update regular expression to limit by start and end, with or without white
 # space
 justify_re <- function(x, whitespace_ok = FALSE) {
-  assert_string(x)
-  assert_flag(whitespace_ok)
-  if (whitespace_ok)
+  stopifnot(is.character(x), length(x) == 1)
+  stopifnot(is.logical(whitespace_ok), length(whitespace_ok) == 1)
+  if (whitespace_ok) {
     paste0("^[[:space:]]*", x, "[[:space:]]*$")
-  else
+  } else {
     paste0("^", x, "$")
+  }
 }
 
 # Allow white space
-justify_re_ws <- function(x)
+.jws <- function(x) {
   justify_re(x, whitespace_ok = TRUE)
+}
 
-# contain any | options within a regular expression, applies to ICD codes
-# without ^ and $
-wrap_re <- function(x)
+# non-capturing expression to contain any | options within a regular expression,
+# applies to ICD codes without ^ and $, (and avoids counting the parenthesis in
+# matched parts)
+nce <- function(x) {
   paste0("(?:", x, ")")
+}
+
+wrap_short_re <- function(maj, min) {
+  paste0(maj, nce(min), "?")
+}
+
+wrap_decimal_re <- function(maj, min) {
+  paste0(
+    maj,
+    nce(
+      paste0(
+        "\\.|",
+        nce(
+          paste0(
+            "\\.",
+            nce(min)
+          )
+        )
+      )
+    ),
+    "?"
+  )
+}
+
+wrap_any_re <- function(maj, min) {
+  paste0(maj, "\\.?", min, "?")
+}
 
 #' Put ICD validation regular expressions in the \code{icd:::} name space
 #' @param env target environment to save the data
 #' @keywords internal sysdata data
+#' @noRd
 set_re_globals <- function(env = parent.frame()) {
-    re_icd9_major_n <- "[[:digit:]]{1,3}"
+  re_icd9_major_n <- "[[:digit:]]{1,3}"
   re_icd9_major_n_strict <- "[[:digit:]]{3}"
   re_icd9_major_v <- "[Vv](?:0[1-9]|[1-9][[:digit:]]?)"
   re_icd9_major_v_strict <- "V(?:0[1-9]|[1-9][[:digit:]])"
   re_icd9_major_e <- "[Ee][[:digit:]]{1,3}"
   re_icd9_major_e_strict <- "E[[:digit:]]{3}"
-  re_icd9_major <- paste0(c(re_icd9_major_n %>% justify_re_ws,
-                            re_icd9_major_v %>% justify_re_ws,
-                            re_icd9_major_e %>% justify_re_ws),
-                          collapse = "|")
-  re_icd9_major_bare <- paste0(c(re_icd9_major_n,
-                                 re_icd9_major_v,
-                                 re_icd9_major_e),
-                               collapse = "|") %>% wrap_re
-  re_icd9_major_strict <- paste0(c(re_icd9_major_n_strict %>% justify_re_ws,
-                                   re_icd9_major_v_strict %>% justify_re_ws,
-                                   re_icd9_major_e_strict %>% justify_re_ws),
-                                 collapse = "|")
-  re_icd9_major_strict_bare <- paste0(c(re_icd9_major_n_strict,
-                                        re_icd9_major_v_strict,
-                                        re_icd9_major_e_strict),
-                                      collapse = "|") %>% wrap_re
+  re_icd9_major <- paste0(c(
+    .jws(re_icd9_major_n),
+    .jws(re_icd9_major_v),
+    .jws(re_icd9_major_e)
+  ),
+  collapse = "|"
+  )
+  re_icd9_major_bare <- nce(paste0(c(
+    re_icd9_major_n,
+    re_icd9_major_v,
+    re_icd9_major_e
+  ),
+  collapse = "|"
+  ))
+  re_icd9_major_strict <- paste0(c(
+    .jws(re_icd9_major_n_strict),
+    .jws(re_icd9_major_v_strict),
+    .jws(re_icd9_major_e_strict)
+  ),
+  collapse = "|"
+  )
+  re_icd9_major_strict_bare <- nce(paste0(c(
+    re_icd9_major_n_strict,
+    re_icd9_major_v_strict,
+    re_icd9_major_e_strict
+  ),
+  collapse = "|"
+  ))
   re_icd9_minor_nv <- "[[:digit:]]{1,2}"
   re_icd9_minor_e <- "[[:digit:]]{1}"
-  re_icd9_decimal_n_bare <- paste0(re_icd9_major_n, "(?:\\.(?:", re_icd9_minor_nv, ")?)?")
-  re_icd9_decimal_v_bare <- paste0(re_icd9_major_v, "(?:\\.(?:", re_icd9_minor_nv, ")?)?")
-  re_icd9_decimal_e_bare <- paste0(re_icd9_major_e, "(?:\\.(?:", re_icd9_minor_e, ")?)?")
-  re_icd9_decimal_n_strict_bare <- paste0(re_icd9_major_n_strict, "(?:\\.(?:", re_icd9_minor_nv, ")?)?")
-  re_icd9_decimal_v_strict_bare <- paste0(re_icd9_major_v_strict, "(?:\\.(?:", re_icd9_minor_nv, ")?)?")
-  re_icd9_decimal_e_strict_bare <- paste0(re_icd9_major_e_strict, "(?:\\.(?:", re_icd9_minor_e, ")?)?")
-  re_icd9_decimal_n <- re_icd9_decimal_n_bare %>% justify_re_ws
-  re_icd9_decimal_v <- re_icd9_decimal_v_bare %>% justify_re_ws
-  re_icd9_decimal_e <- re_icd9_decimal_e_bare %>% justify_re_ws
-  re_icd9_short_n <- paste0(re_icd9_major_n, "(?:", re_icd9_minor_nv, ")?") %>% justify_re_ws
-  re_icd9_short_v <- paste0(re_icd9_major_v, "(?:", re_icd9_minor_nv, ")?") %>% justify_re_ws
-  re_icd9_short_e <- paste0(re_icd9_major_e, "(?:", re_icd9_minor_e, ")?") %>% justify_re_ws
-  re_icd9_any_n <- paste0(re_icd9_major_n, "\\.?(?:", re_icd9_minor_nv, ")?") %>% justify_re_ws
-  re_icd9_any_v <- paste0(re_icd9_major_v, "\\.?(?:", re_icd9_minor_nv, ")?") %>% justify_re_ws
-  re_icd9_any_e <- paste0(re_icd9_major_e, "\\.?(?:", re_icd9_minor_e, ")?") %>% justify_re_ws
-  re_icd9_decimal <- paste0(c(re_icd9_decimal_n,
-                              re_icd9_decimal_v,
-                              re_icd9_decimal_e),
-                            collapse = "|")
-  re_icd9_decimal_bare <- paste0(c(re_icd9_decimal_n_bare,
-                                   re_icd9_decimal_v_bare,
-                                   re_icd9_decimal_e_bare),
-                                 collapse = "|")
-  re_icd9_decimal_strict_bare <- paste0(c(re_icd9_decimal_n_strict_bare,
-                                          re_icd9_decimal_v_strict_bare,
-                                          re_icd9_decimal_e_strict_bare),
-                                        collapse = "|")
-  re_icd9_short <- paste0(c(re_icd9_short_n,
-                            re_icd9_short_v,
-                            re_icd9_short_e),
-                          collapse = "|")
-  re_icd9_any <- paste0(c(re_icd9_any_n,
-                          re_icd9_any_v,
-                          re_icd9_any_e),
-                        collapse = "|")
-  re_icd10cm_major_bare  <- "[[:alpha:]][[:digit:]][[:alnum:]]"
-  re_icd10_major_bare <- re_icd10cm_major_bare # use slightly broader definition for generic
-  re_icd10cm_major  <- re_icd10cm_major_bare %>% justify_re_ws
-  re_icd10_major <- re_icd10_major_bare %>% justify_re_ws
-  re_icd10cm_short <- paste0(re_icd10_major_bare, "[[:alnum:]]{0,4}")
-  re_icd10cm_decimal <- paste0(re_icd10_major_bare, "(?:\\.[[:alnum:]]{0,4})?")
-  re_icd10cm_any <- paste0(re_icd10_major_bare, "\\.?[[:alnum:]]{0,4}")
-  re_icd10_short <- re_icd10cm_short
-  re_icd10_decimal <- re_icd10cm_decimal
-  re_icd10_any <- re_icd10cm_any
+  re_icd9_decimal_n_bare <- paste0(
+    re_icd9_major_n, "(?:\\.(?:",
+    re_icd9_minor_nv, ")?)?"
+  )
+  re_icd9_decimal_v_bare <-
+    wrap_decimal_re(re_icd9_major_v, re_icd9_minor_nv)
+  re_icd9_decimal_e_bare <-
+    wrap_decimal_re(re_icd9_major_e, re_icd9_minor_e)
+  re_icd9_decimal_n_strict_bare <-
+    wrap_decimal_re(re_icd9_major_n_strict, re_icd9_minor_nv)
+  re_icd9_decimal_v_strict_bare <-
+    wrap_decimal_re(re_icd9_major_v_strict, re_icd9_minor_nv)
+  re_icd9_decimal_e_strict_bare <-
+    wrap_decimal_re(re_icd9_major_e_strict, re_icd9_minor_e)
+  re_icd9_decimal_n <- .jws(re_icd9_decimal_n_bare)
+  re_icd9_decimal_v <- .jws(re_icd9_decimal_v_bare)
+  re_icd9_decimal_e <- .jws(re_icd9_decimal_e_bare)
+  re_icd9_short_n <- .jws(wrap_short_re(re_icd9_major_n, re_icd9_minor_nv))
+  re_icd9_short_v <- .jws(wrap_short_re(re_icd9_major_v, re_icd9_minor_nv))
+  re_icd9_short_e <- .jws(wrap_short_re(re_icd9_major_e, re_icd9_minor_e))
+  re_icd9_any_n <- .jws(wrap_any_re(re_icd9_major_n, re_icd9_minor_nv))
+  re_icd9_any_v <- .jws(wrap_any_re(re_icd9_major_v, re_icd9_minor_nv))
+  re_icd9_any_e <- .jws(wrap_any_re(re_icd9_major_e, re_icd9_minor_e))
+  re_icd9_decimal <- paste0(c(
+    re_icd9_decimal_n,
+    re_icd9_decimal_v,
+    re_icd9_decimal_e
+  ),
+  collapse = "|"
+  )
+  re_icd9_decimal_bare <- paste0(c(
+    re_icd9_decimal_n_bare,
+    re_icd9_decimal_v_bare,
+    re_icd9_decimal_e_bare
+  ),
+  collapse = "|"
+  )
+  re_icd9_decimal_strict_bare <- paste0(c(
+    re_icd9_decimal_n_strict_bare,
+    re_icd9_decimal_v_strict_bare,
+    re_icd9_decimal_e_strict_bare
+  ),
+  collapse = "|"
+  )
+  re_icd9_short <- paste0(c(
+    re_icd9_short_n,
+    re_icd9_short_v,
+    re_icd9_short_e
+  ),
+  collapse = "|"
+  )
+  re_icd9_any <- paste0(c(
+    re_icd9_any_n,
+    re_icd9_any_v,
+    re_icd9_any_e
+  ),
+  collapse = "|"
+  )
+  re_icd10_major_bare <- "[[:alpha:]][[:digit:]][[:alnum:]]"
+  # use slightly broader definition for generic:
+  re_icd10_major <- .jws(re_icd10_major_bare)
+  re_icd10_minor <- "[[:alnum:]]{0,4}"
+  re_icd10_short <- paste0(re_icd10_major_bare, re_icd10_minor)
+  re_icd10_decimal <-
+    wrap_decimal_re(re_icd10_major_bare, re_icd10_minor)
+  re_icd10_any <- wrap_any_re(re_icd10_major_bare, re_icd10_minor)
+  # ICD-10-CM, ICD-10-BE
+  re_icd10cm_major_bare <- re_icd10_major_bare
+  re_icd10cm_major <- .jws(re_icd10cm_major_bare)
+  re_icd10cm_minor <- re_icd10_minor
+  re_icd10cm_short <- paste0(re_icd10cm_major_bare, re_icd10cm_minor)
+  re_icd10cm_decimal <-
+    wrap_decimal_re(re_icd10cm_major_bare, re_icd10cm_minor)
+  re_icd10cm_any <- wrap_any_re(re_icd10cm_major_bare, re_icd10cm_minor)
+  # ICD-10-FR see cim.pdf in the zip from <https://www.atih.sante.fr/plateformes-de-transmission-et-logiciels/logiciels-espace-de-telechargement/telecharger/gratuit/11616/456>
+  re_icd10fr_major_bare <- re_icd10_major_bare
+  re_icd10fr_major <- .jws(re_icd10fr_major_bare)
+  re_icd10fr_minor <- "[[:digit:]+]{1,2}[[:digit:]]?"
+  re_icd10fr_short <- wrap_short_re(re_icd10fr_major_bare, re_icd10fr_minor)
+  re_icd10fr_decimal <- wrap_decimal_re(re_icd10fr_major_bare, re_icd10fr_minor)
+  re_icd10fr_any <- wrap_any_re(re_icd10fr_major_bare, re_icd10fr_minor)
   cur_env <- environment()
-  for (re in ls(envir = cur_env, pattern = "re_.+"))
+  for (re in ls(envir = cur_env, pattern = "re_.+")) {
     assign(re, get(re, envir = cur_env), envir = env)
-  list(re_icd10_any, re_icd10_decimal, re_icd10_short, re_icd10cm_any,
-       re_icd10cm_decimal, re_icd10cm_short, re_icd10_major,
-       re_icd10cm_major, re_icd10_major_bare,
-       re_icd10cm_major_bare, re_icd9_any, re_icd9_short, re_icd9_decimal_strict_bare,
-       re_icd9_decimal_bare, re_icd9_decimal, re_icd9_any_e, re_icd9_any_v,
-       re_icd9_any_n, re_icd9_short_e, re_icd9_short_v, re_icd9_short_n,
-       re_icd9_decimal_e, re_icd9_decimal_v, re_icd9_decimal_n,
-       re_icd9_decimal_e_strict_bare, re_icd9_decimal_v_strict_bare, re_icd9_decimal_n_strict_bare,
-       re_icd9_decimal_e_bare, re_icd9_decimal_v_bare, re_icd9_decimal_n_bare, re_icd9_minor_e,
-       re_icd9_minor_nv, re_icd9_major_strict_bare, re_icd9_major_strict, re_icd9_major_bare,
-       re_icd9_major, re_icd9_major_e_strict, re_icd9_major_e, re_icd9_major_v_strict,
-       re_icd9_major_v, re_icd9_major_n_strict, re_icd9_major_n)
+  }
+  list(
+    re_icd10_any,
+    re_icd10cm_any,
+    re_icd10fr_any,
+    re_icd10_decimal,
+    re_icd10cm_decimal,
+    re_icd10fr_decimal,
+    re_icd10_short,
+    re_icd10cm_short,
+    re_icd10fr_short,
+    re_icd10_major,
+    re_icd10cm_major,
+    re_icd10fr_major,
+    re_icd10_minor,
+    re_icd10cm_minor,
+    re_icd10fr_minor,
+    re_icd10_major_bare,
+    re_icd10cm_major_bare,
+    re_icd10fr_major_bare,
+    re_icd9_any,
+    re_icd9_short,
+    re_icd9_decimal_strict_bare,
+    re_icd9_decimal_bare,
+    re_icd9_decimal,
+    re_icd9_any_e,
+    re_icd9_any_v,
+    re_icd9_any_n,
+    re_icd9_short_e,
+    re_icd9_short_v,
+    re_icd9_short_n,
+    re_icd9_decimal_e,
+    re_icd9_decimal_v,
+    re_icd9_decimal_n,
+    re_icd9_decimal_e_strict_bare,
+    re_icd9_decimal_v_strict_bare,
+    re_icd9_decimal_n_strict_bare,
+    re_icd9_decimal_e_bare,
+    re_icd9_decimal_v_bare,
+    re_icd9_decimal_n_bare,
+    re_icd9_minor_e,
+    re_icd9_minor_nv,
+    re_icd9_major_strict_bare,
+    re_icd9_major_strict,
+    re_icd9_major_bare,
+    re_icd9_major,
+    re_icd9_major_e_strict,
+    re_icd9_major_e,
+    re_icd9_major_v_strict,
+    re_icd9_major_v,
+    re_icd9_major_n_strict,
+    re_icd9_major_n
+  )
 }
 # and put these in the package namespace
 set_re_globals()
@@ -184,24 +318,38 @@ set_re_globals()
 #' @return logical vector with \code{TRUE} or \code{FALSE} for each ICD code
 #'   provided according to its validity
 #' @examples
-#'   is_valid(as.icd9(c("", "1", "22", "333", "4444", "123.45", "V",
-#'                      "V2", "V34", "V567", "E", "E1", "E70", "E")))
-#'   # internal function:
-#'   icd:::is_valid_major(c("", "1", "22", "333", "4444", "123.45", "V",
-#'                      "V2", "V34", "V567", "E", "E1", "E70", "E"))
+#' is_valid(as.icd9(c(
+#'   "", "1", "22", "333", "4444", "123.45", "V",
+#'   "V2", "V34", "V567", "E", "E1", "E70", "E"
+#' )))
+#' # internal function:
+#' icd:::is_valid_major(c(
+#'   "", "1", "22", "333", "4444", "123.45", "V",
+#'   "V2", "V34", "V567", "E", "E1", "E70", "E"
+#' ))
 #' @export
-is_valid <- function(x, ...)
+is_valid <- function(x, ...) {
   UseMethod("is_valid")
+}
 
-icd_valid_worker <- function(x, whitespace_ok, regex, regex_no_ws = regex) {
+icd_valid_worker <- function(x,
+                             whitespace_ok,
+                             regex,
+                             regex_no_ws = regex) {
   assert_flag(whitespace_ok)
-  if (length(x) == 0)
+  if (length(x) == 0) {
     return(logical())
-  assert(check_factor(x), check_character(x))
-  if (whitespace_ok)
-    na_to_false(grepl(justify_re_ws(regex), x, perl = TRUE))
-  else
-    na_to_false(grepl(justify_re(regex_no_ws), x, perl = TRUE))
+  }
+  assert_fac_or_char(x)
+  if (whitespace_ok) {
+    na_to_false(
+      grepl(pattern = .jws(regex), x = x, perl = TRUE)
+    )
+  } else {
+    na_to_false(
+      grepl(pattern = justify_re(regex_no_ws), x = x, perl = TRUE)
+    )
+  }
 }
 
 #' @describeIn is_valid_major Test whether an ICD code is of major type,
@@ -222,51 +370,82 @@ is_valid.default <- function(x, short_code = guess_short(x), ...) {
 #' @export
 #' @keywords internal
 is_valid.icd10 <- function(x, short_code = guess_short(x),
-                           whitespace_ok = TRUE, ...) {
+                           whitespace_ok = TRUE,
+                           re_short = re_icd10_short,
+                           re_decimal = re_icd10_decimal,
+                           ...) {
   assert_flag(short_code)
   # SOMEDAY: check whether code has 'year' attribute. This is maybe more for
   # testing 'realness' start with a broad regular expression
-  if (short_code)
-    grepl(pattern = justify_re(re_icd10_short), trim(x), perl = TRUE)
-  else
-    grepl(pattern = justify_re(re_icd10_decimal), trim(x), perl = TRUE)
+  if (short_code) {
+    grepl(pattern = justify_re(re_short), trimws(x), perl = TRUE)
+  } else {
+    grepl(pattern = justify_re(re_decimal), trimws(x), perl = TRUE)
+  }
+}
+
+#' @describeIn is_valid Test whether generic ICD-10-FR code is valid
+#' @export
+#' @keywords internal
+is_valid.icd10fr <- function(x,
+                             short_code = guess_short(x),
+                             whitespace_ok = TRUE,
+                             ...) {
+  is_valid.icd10(
+    x = x,
+    short_code = short_code,
+    whitespace_ok = whitespace_ok,
+    re_short = re_icd10fr_short,
+    re_decimal = re_icd10fr_decimal,
+    ...
+  )
 }
 
 #' @describeIn is_valid Test whether generic ICD-10 code is valid
 #' @export
 #' @keywords internal
-is_valid.icd9 <- function(x, short_code = guess_short(x),
-                          whitespace_ok = TRUE, ...) {
-  assert_flag(short_code)
-  if (short_code)
+is_valid.icd9 <- function(x,
+                          short_code = guess_short(x),
+                          whitespace_ok = TRUE,
+                          ...) {
+  if (short_code) {
     icd9_is_valid_short(x, whitespace_ok = whitespace_ok)
-  else
+  } else {
     icd9_is_valid_decimal(x, whitespace_ok = whitespace_ok)
+  }
 }
 
-icd9_is_valid_decimal <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_decimal <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_decimal)
+}
 
-icd9_is_valid_short <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_short <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_short)
+}
 
-icd9_is_valid_short_n <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_short_n <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_short_n)
+}
 
-icd9_is_valid_short_v <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_short_v <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_short_v)
+}
 
-icd9_is_valid_short_e <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_short_e <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_short_e)
+}
 
-icd9_is_valid_decimal_n <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_decimal_n <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_decimal_n)
+}
 
-icd9_is_valid_decimal_v <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_decimal_v <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_decimal_v)
+}
 
-icd9_is_valid_decimal_e <- function(x, whitespace_ok = TRUE)
+icd9_is_valid_decimal_e <- function(x, whitespace_ok = TRUE) {
   icd_valid_worker(x, whitespace_ok, re_icd9_decimal_e)
+}
 
 #' Test whether an ICD code is major
 #'
@@ -320,8 +499,10 @@ icd9_is_valid_major_e <- function(x, whitespace_ok = TRUE)
 #' @export
 #' @keywords internal
 is_valid.comorbidity_map <- function(x, short_code, ...) {
-  assert_list(x, types = "character",
-              min.len = 1, unique = TRUE, names = "named")
+  assert_list(x,
+    types = "character",
+    min.len = 1, unique = TRUE, names = "named"
+  )
   assert_flag(short_code)
   all(unlist(
     lapply(x, FUN = function(y) is_valid(y, short_code = short_code)),
@@ -342,7 +523,8 @@ is_valid.comorbidity_map <- function(x, short_code, ...) {
 get_valid <- function(x, short_code = guess_short(x))
   UseMethod("get_valid")
 
-#' @describeIn get_valid get valid ICD codes from character vector, guessing ICD version
+#' @describeIn get_valid get valid ICD codes from character vector, guessing ICD
+#'   version
 #' @export
 #' @keywords internal
 get_valid.character <- function(x, short_code = guess_short(x)) {
@@ -416,8 +598,7 @@ get_invalid.icd10 <- function(x, short_code = guess_short(x), ...) {
 #' @keywords internal
 get_invalid.comorbidity_map <- function(x, short_code = guess_short(x), ...) {
   class(x) <- class(x)[class(x) != "comorbidity_map"]
-  x <- lapply(x, FUN = get_invalid, short_code = short_code)
-  x[lapply(x, length) > 0]
+  Filter(length, lapply(x, FUN = get_invalid, short_code = short_code))
 }
 
 #' Get major part of an ICD code
@@ -425,34 +606,79 @@ get_invalid.comorbidity_map <- function(x, short_code = guess_short(x), ...) {
 #' Returns major component of codes, i.e. the part before the decimal,
 #' or where the decimal would be.
 #' @keywords internal
+#' @noRd
 get_major <- function(x)
   UseMethod("get_major")
 
-#' @describeIn get_major Get major part of an ICD-10 code
+#' @describeIn get_major Get major part of an ICD-10 (of any kind) code
 #' @details For ICD-10, this is an initial implementation. If speed needed, then
 #'   can re-use C++ ICD-9 version: just grabbing the first three characters,
 #'   after all, and this is much easier in ICD-10 then ICD-9
 #' @keywords internal
 #' @export
-get_major.icd10 <- function(x)
-  substr(trim(x), 1L, 3L)
+#' @noRd
+get_major.icd10 <- function(x) {
+  substr(trimws(x), 1L, 3L)
+}
+
+#' @describeIn get_major Get major part of an ICD-10-CM code
+#' @keywords internal
+#' @export
+#' @noRd
+get_major.icd10cm <- function(x) {
+  get_major.icd10(x)
+}
+
+#' @describeIn get_major Get major part of a French (country, not language) ICD-10-FR code
+#' @keywords internal
+#' @export
+#' @noRd
+get_major.icd10fr <- function(x) {
+  get_major.icd10(x)
+}
+
+#' @describeIn get_major Get major part of a Belgian ICD-10-BE code
+#' @keywords internal
+#' @export
+#' @noRd
+get_major.icd10be <- function(x) {
+  get_major.icd10(x)
+}
+
+#' @describeIn get_major Get major part of a WHO ICD-10 code
+#' @keywords internal
+#' @export
+#' @noRd
+get_major.icd10who <- function(x) {
+  get_major.icd10(x)
+}
+
+#' @describeIn get_major Get major part of an ICD-10-CM procedure code
+#' @keywords internal
+#' @noRd
+#' @export
+get_major.icd10cm_pc <- function(x) {
+  substr(trimws(x), 1L, 3L)
+}
 
 #' Check whether a code is major
 #' @param icd character vector of ICD codes.
 #' @keywords internal
+#' @noRd
 is_major <- function(x)
   UseMethod("is_major")
 
 #' @describeIn is_major Default method which guesses version
 #' @keywords internal
 #' @export
+#' @noRd
 is_major.default <- function(x) {
   switch(
     guess_version(x),
     "icd9" = is_major.icd9(x),
     "icd9cm" = is_major.icd9(x),
-    "icd10" = is_major.icd10(x),
-    "icd10cm" = is_major.icd10(x),
+    "icd10" = is_major.icd10cm(x),
+    "icd10cm" = is_major.icd10cm(x),
     stop("ICD version not known")
   )
 }
@@ -460,6 +686,7 @@ is_major.default <- function(x) {
 #' @describeIn is_major check whether a code is an ICD-10 major
 #' @keywords internal
 #' @export
+#' @noRd
 is_major.icd10 <- function(x) {
   assert_character(x)
   # if not know whether ICD-10-CM, then use broader definition
@@ -471,16 +698,18 @@ is_major.icd10 <- function(x) {
 #'   quite fast, but does suffer from handling Unicode, locales, etc.
 #' @keywords internal
 #' @export
+#' @noRd
 is_major.icd10cm <- function(x) {
   assert_character(x)
-  grepl(justify_re_ws(re_icd10cm_major), trim(x), perl = TRUE)
+  grepl(re_icd10cm_major, trimws(x), perl = TRUE)
 }
 
 #' @describeIn is_major check whether a code is an ICD-9 major
 #' @keywords internal
 #' @export
+#' @noRd
 is_major.icd9 <- function(x) {
-  x <- trim(x)
+  x <- sub(pattern = "\\.", replacement = "", x)
   nchar(x) - icd9_is_e(x) < 4
 }
 
@@ -491,15 +720,18 @@ is_major.icd9 <- function(x) {
 #' @param x vector of strings or factor to test
 #' @return logical vector
 #' @keywords internal
+#' @noRd
 icd9_is_n <- function(x)
-  icd9_is_n_cpp(as_char_no_warn(x))
+  icd9_is_n_rcpp(as_char_no_warn(x))
 
 #' @describeIn icd9_is_n are the given codes V type?
 #' @keywords internal
+#' @noRd
 icd9_is_v <- function(x)
-  icd9_is_v_cpp(as_char_no_warn(x))
+  icd9_is_v_rcpp(as_char_no_warn(x))
 
 #' @describeIn icd9_is_n are the given codes E type?
 #' @keywords internal
+#' @noRd
 icd9_is_e <- function(x)
-  icd9_is_e_cpp(as_char_no_warn(x))
+  icd9_is_e_rcpp(as_char_no_warn(x))
