@@ -47,7 +47,6 @@ test_that("well ordered class lists short/decimal ICD combos are created", {
 })
 
 test_that("is short or decimal code", {
-  # todo, somehow clarify that this is not the same is is_valid...
   expect_true(is.short_diag(as.short_diag("1234")))
   expect_true(is.decimal_diag(as.decimal_diag("12.34")))
   expect_true(is.short_diag(as.short_diag("1234")))
@@ -76,7 +75,7 @@ test_that("attributes set and queried", {
   attr(y, "icd_short_diag") <- TRUE
   attr(z, "icd_short_diag") <- FALSE
   expect_true(is.short_diag(x))
-  expect_null(is.short_diag(no_short_code))
+  expect_false(is.short_diag(no_short_code))
 })
 
 test_that("ICD version supertype set", {
@@ -95,19 +94,15 @@ test_that("constructing a comorbidity map works", {
 
 test_that("constructing a comorbidity map with unnamed list, etc. fails", {
   expect_error(as.comorbidity_map(unname(unclass(x))))
-  # and data frames should definitely fail
-  expect_error(as.comorbidity_map(vermont_dx))
-  expect_error(as.comorbidity_map(uranium_pathology))
 })
 
 test_that("subsetting a comorbidity map gives the right class", {
-
   wonky_map <- comorbidity_map(list(a = as.icd9cm("100"), b = as.icd9cm("V22")))
   attr(wonky_map$a, "icd_short_diag") <- TRUE
   attr(wonky_map$b, "icd_short_diag") <- TRUE
 
   expect_is(wonky_map, "comorbidity_map")
-  expect_null(is.short_diag(wonky_map))
+  expect_false(is.short_diag(wonky_map))
 
   expect_true(!inherits(wonky_map[[1]], "comorbidity_map"))
   expect_true(is.short_diag(wonky_map[[1]]))
@@ -129,15 +124,35 @@ test_that("subsetting a comorbidity map gives the right class", {
 })
 
 test_that("constructing wide data works", {
-  expect_equal(as.icd_wide_data(vermont_dx), vermont_dx)
-  expect_equivalent(as.icd_wide_data(vermont_dx), vermont_dx)
-  expect_equivalent(as.icd_wide_data(as.data.frame(vermont_dx)), vermont_dx)
+  expect_equal(
+    as.icd_wide_data(vermont_dx),
+    vermont_dx
+  )
+  expect_equivalent(
+    as.icd_wide_data(vermont_dx),
+    vermont_dx
+  )
+  expect_equivalent(
+    as.icd_wide_data(as.data.frame(vermont_dx)),
+    vermont_dx
+  )
 })
 
 test_that("constructing long data works", {
-  expect_equal(as.icd_long_data(uranium_pathology), uranium_pathology)
-  expect_equivalent(as.icd_long_data(uranium_pathology), uranium_pathology)
-  expect_equivalent(as.icd_long_data(as.data.frame(uranium_pathology)), uranium_pathology)
+  l <- data.frame(id = c(1, 1), code = c("A10", "B10"))
+  expect_true(is.icd_long_data(as.icd_long_data(l)))
+  expect_identical(
+    as.icd_long_data(uranium_pathology),
+    uranium_pathology
+  )
+  expect_equivalent(
+    as.icd_long_data(uranium_pathology),
+    uranium_pathology
+  )
+  expect_identical(
+    as.icd_long_data(as.data.frame(uranium_pathology)),
+    uranium_pathology
+  )
 })
 
 test_that("is long or wide data?", {
@@ -155,24 +170,42 @@ test_that("constructing wide or long format for non-data frame gives error", {
 })
 
 test_that("subsetting data frame works", {
-  expect_equal(unclass(vermont_dx[1, 6]), as.short_diag("27801"))
-  expect_equal(unclass(vermont_dx[1, "DX1"]), as.short_diag("27801"))
-  expect_is(vermont_dx[1, "DX1"], c("icd9cm", "icd9", "character"))
-  expect_equivalent(unclass(vermont_dx[[1, 6]]), "27801")
-  expect_is(vermont_dx[[1, "DX1"]], c("icd9cm", "icd9", "character"))
-  expect_true(is.icd9(vermont_dx[[1, "DX9"]]))
-  expect_true(is.icd9cm(vermont_dx[[1, "DX12"]]))
-  # columns
-  expect_is(vermont_dx[6], c("icd9cm", "icd9", "data.frame")) # not necessarily wide anymore...
-  expect_is(vermont_dx[[6]], c("icd9cm", "icd9", "character"))
+  v_subsets <- list(
+    vermont_dx,
+    vermont_dx[1:10, ],
+    vermont_dx[, 1:15],
+    vermont_dx[1:10, 1:17]
+  )
+  for (v in v_subsets) {
+    i <- paste(nrow(v), "x", ncol(v))
+    expect_equal(unclass(v[1, "DX1"]), as.short_diag("27801"), info = i)
+    expect_true(is.short_diag(v[, "DX2"]))
+    expect_equivalent(unclass(v[[1, "DX1"]]), "27801", info = i)
+    expect_is(v[2, "DX3"], c("icd9cm", "icd9", "character"), info = i)
+    expect_is(v[[3, "DX4"]], c("icd9cm", "icd9", "character"), info = i)
+    expect_true(is.icd9(v[[4, "DX5"]]), info = i)
+    expect_true(is.icd9cm(v[[5, "DX6"]]), info = i)
+    # columns
+    expect_is(v[9], c("icd9cm", "icd9", "data.frame"), info = i)
+    expect_is(v[[8]], c("icd9cm", "icd9", "character"), info = i)
+  }
 })
 
 test_that("data frame subsetting doesn't incorrectly set class on columns", {
-  expect_numeric(pts_invalid_mix[c(TRUE, TRUE, TRUE), "visit_id"])
-  expect_false(inherits(pts_invalid_mix[c(TRUE, TRUE, TRUE), "visit_id"], "icd9"))
-  expect_equal(sapply(pts_invalid_mix[c(TRUE, TRUE, TRUE), ], class),
-               structure(list(visit_id = "numeric", icd9 = c("icd9", "character"
-               ), poa = "factor"), .Names = c("visit_id", "icd9", "poa"))
+  expect_true(is.numeric(pts_invalid_mix[c(TRUE, TRUE, TRUE), "visit_id"]))
+  expect_false(
+    inherits(pts_invalid_mix[c(TRUE, TRUE, TRUE), "visit_id"], "icd9")
+  )
+  expect_equal(
+    lapply(pts_invalid_mix[c(TRUE, TRUE, TRUE), ], class),
+    structure(
+      list(
+        visit_id = "numeric",
+        icd9 = c("icd9", "character"),
+        poa = "factor"
+      ),
+      .Names = c("visit_id", "icd9", "poa")
+    )
   )
 })
 
@@ -180,13 +213,15 @@ test_that("printing a comorbidity map works very simply", {
   expect_warning(
     capture.output(
       print.comorbidity_map(icd9_map_quan_elix)
-    ), regexp = NA)
+    ),
+    regexp = NA
+  )
 })
 
 test_that("is comorbidity map?", {
   icd9_map_ahrq %>%
-    unclass %>%
-    comorbidity_map -> x
+    unclass() %>%
+    comorbidity_map() -> x
   expect_true(is.comorbidity_map(icd9_map_ahrq))
   expect_true(is.comorbidity_map(x))
   expect_is(icd9_map_ahrq, "comorbidity_map")
@@ -197,22 +232,20 @@ context("class conflicts")
 
 test_that("no conflict for standard classes", {
   expect_false(icd_classes_conflict(icd9("100.1")))
-  expect_false(icd_classes_conflict(icd9("100.1") %>% as.decimal_diag))
-  expect_false(icd_classes_conflict(icd9("1001") %>% as.short_diag))
+  expect_false(icd_classes_conflict(icd9("100.1") %>% as.decimal_diag()))
+  expect_false(icd_classes_conflict(icd9("1001") %>% as.short_diag()))
   expect_false(icd_classes_conflict(icd9cm("100.1")))
-  expect_false(icd_classes_conflict(icd9cm("100.1") %>% as.decimal_diag))
-  expect_false(icd_classes_conflict(icd9cm("1001") %>% as.short_diag))
+  expect_false(icd_classes_conflict(icd9cm("100.1") %>% as.decimal_diag()))
+  expect_false(icd_classes_conflict(icd9cm("1001") %>% as.short_diag()))
   expect_false(icd_classes_conflict(icd10("A00.0")))
-  expect_false(icd_classes_conflict(icd10("A00.0") %>% as.decimal_diag))
-  expect_false(icd_classes_conflict(icd10("A000") %>% as.short_diag))
+  expect_false(icd_classes_conflict(icd10("A00.0") %>% as.decimal_diag()))
+  expect_false(icd_classes_conflict(icd10("A000") %>% as.short_diag()))
   expect_false(icd_classes_conflict(icd10cm("A00.0")))
-  expect_false(icd_classes_conflict(icd10cm("A00.0") %>% as.decimal_diag))
-  expect_false(icd_classes_conflict(icd10cm("A000") %>% as.short_diag))
+  expect_false(icd_classes_conflict(icd10cm("A00.0") %>% as.decimal_diag()))
+  expect_false(icd_classes_conflict(icd10cm("A000") %>% as.short_diag()))
 })
 
 test_that("no conflict for built-in data", {
-  expect_false(icd_classes_conflict(vermont_dx))
-  expect_false(icd_classes_conflict(uranium_pathology))
   expect_false(icd_classes_conflict(icd9_map_elix))
   expect_false(icd_classes_conflict(icd9_map_elix[2]))
   expect_false(icd_classes_conflict(icd9_map_elix[[2]]))
@@ -221,25 +254,67 @@ test_that("no conflict for built-in data", {
 })
 
 test_that("conflicting ICD type classes can be found", {
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd9cm", "icd10", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd9", "icd10", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd9cm", "icd10cm", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd10cm", "icd9", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd10cm", "icd9cm", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd10", "icd9", "character"))))
-  expect_true(icd_classes_conflict(structure("V10", class = c("icd10", "icd9cm", "character"))))
-  expect_true(icd_classes_conflict(structure(list("V10", "A20"), class = c("icd9cm", "icd10", "list"))))
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd9cm", "icd10", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd9", "icd10", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd9cm", "icd10cm", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd10cm", "icd9", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd10cm", "icd9cm", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd10", "icd9", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure("V10",
+        class = c("icd10", "icd9cm", "character")
+      )
+    )
+  )
+  expect_true(
+    icd_classes_conflict(
+      structure(list("V10", "A20"),
+        class = c("icd9cm", "icd10", "list")
+      )
+    )
+  )
 })
 
 test_that("we can't set a data.frame to have a vector class", {
-  expect_error(as.icd9(data.frame()))
-  expect_error(as.icd9cm(data.frame()))
-  expect_error(as.icd10(data.frame()))
-  expect_error(as.icd10cm(data.frame()))
-
   expect_error(as.icd9(vermont_dx))
   expect_error(as.icd9cm(vermont_dx))
-
   expect_error(as.icd10(uranium_pathology))
   expect_error(as.icd10cm(uranium_pathology))
 })
@@ -249,17 +324,48 @@ test_that("long vs wide data conflict identified", {
   class(v_bad) <- c(class(v_bad), "icd_long_data")
   u_bad <- uranium_pathology
   class(u_bad) <- c(class(u_bad), "icd_wide_data")
-
   expect_true(icd_classes_conflict(v_bad))
   expect_true(icd_classes_conflict(u_bad))
 })
 
-
-context("class updates")
-
 test_that("can create NA valued ICD code types", {
-  checkmate::expect_scalar_na(as.icd9(NA))
-  checkmate::expect_scalar_na(as.icd9cm(NA))
-  checkmate::expect_scalar_na(as.icd10(NA))
-  checkmate::expect_scalar_na(as.icd10cm(NA))
+  expect_true(is.na(as.icd9(NA)))
+  expect_true(is.na(as.icd9cm(NA)))
+  expect_true(is.na(as.icd10(NA)))
+  expect_true(is.na(as.icd10cm(NA)))
+})
+test_that("icd9 procedure code classes look good", {
+  expect_false(icd_conflicts.icd9cm_pc(as.icd9cm_pc("0016073")))
+  expect_true(icd_conflicts.icd9cm_pc(as.icd9("A00")))
+  expect_true(icd_conflicts.icd9cm_pc(as.icd9cm("A00")))
+  expect_identical(icd9cm_pc("001"), as.icd9cm_pc("001"))
+  expect_error(as.icd9cm_pc(as.icd9cm("A00")))
+})
+test_that("icd10 procedure code classes look good", {
+  expect_false(icd_conflicts.icd10cm_pc(as.icd10cm_pc("0016073")))
+  expect_true(icd_conflicts.icd10cm_pc(as.icd10("A00")))
+  expect_true(icd_conflicts.icd10cm_pc(as.icd10cm("A00")))
+  expect_true(icd_conflicts.icd10cm_pc(as.icd10who("A00")))
+  expect_identical(icd10cm_pc("001"), as.icd10cm_pc("001"))
+  expect_true(is.icd10cm_pc(icd10cm_pc("001")))
+  expect_true(is.icd10cm_pc(as.icd10cm_pc("001")))
+  expect_error(as.icd10cm_pc(as.icd10cm("A00")))
+})
+test_that("icd10 WHO code classes look good", {
+  expect_false(icd_conflicts.icd10who(as.icd10who("A00")))
+  expect_true(icd_conflicts.icd10cm_pc(as.icd10("A00")))
+  expect_true(icd_conflicts.icd10who(as.icd10cm_pc("001")))
+  expect_true(icd_conflicts.icd10cm(as.icd10who("A00")))
+  expect_true(is.icd10who(icd10who("Z998")))
+  expect_true(is.icd10who(as.icd10who("Z998")))
+  expect_false(is.icd10who(as.icd10cm("Z998")))
+})
+test_that("icd10-fr France code classes look good", {
+  expect_false(icd_conflicts.icd10fr(as.icd10fr("A00")))
+  expect_true(icd_conflicts.icd10cm_pc(as.icd10("A00")))
+  expect_true(icd_conflicts.icd10fr(as.icd10cm_pc("001")))
+  expect_true(icd_conflicts.icd10fr(as.icd10who("A00")))
+  expect_true(is.icd10fr(icd10fr("Z998")))
+  expect_true(is.icd10fr(as.icd10fr("Z998")))
+  expect_false(is.icd10fr(as.icd10cm("Z998")))
 })
